<?php
namespace Controller;

use Util\InstanceProvider;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource;
use Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization;
use \Service\InitService;
use \Util\Utilities;
use \Util\Messages;


class InitController extends Simple {

	private $kernel;
	private $projectId;
	private $initService;
	private $entityManager;
	private $environment;
	protected $resourceProjectService;

	public function __construct(Kernel $kernel, InitService $initService, EntityManager $entityManager, Utilities $environment,
								\Service\Resource $resourceService, $projectId){
		$this->kernel = $kernel;
		$this->initService = $initService;
		$this->entityManager = $entityManager;
		$this->environment = $environment;
		$this->resourceService = $resourceService;
		$this->projectId = $projectId;
	}

    public function findTaskOptionsParameters() {
        return InstanceProvider::getRepository()->getTask()->getTaskOptionsParameters($this->environment->getCurrentOrganizationId())[0];
    }

	public function startSession(Request $request, Response $response) {
		try {
			$userInfo = $this->initService->restoreSession();

            $organizationId = $userInfo['ORGANIZATION_ID'];
            $organizationName = $userInfo['ORGANIZATION_NAME'];
            $userId = $userInfo['USER_ID'];
            $userName = $userInfo['USER_NAME'];
            $userEmail = $userInfo['USER_EMAIL'];

            $this->initService->checkUserRecords($organizationId,$organizationName,$userId,$userName,$userEmail);

            $data = array(
                "USER_NAME"         => $userName,
                "USER_EMAIL"        => $userEmail,
                "ORGANIZATION_NAME" => $organizationName,
                "ORGANIZATION_ID"   => $organizationId,
                "EXTERNAL_ORG_ID"   => $organizationId,
                "USER_ID"           => $userId,
                'DEFAULT_ACL_RULE'  => $this->getDefaultAclRule(),
                "TASK_PARAMETERS"   => $this->findTaskOptionsParameters(),
            );

			$response->addDataSet(new DataSet('system', $data));
		} catch(\Exception $e) {
			$response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
			throw $e;
		}
	}

	public function logout(Request $request, Response $response) {
		try {
			$httpRequest = $this->kernel->getHttpRequest();
			$token = $httpRequest->get('TOKEN');
			$this->initService->closeSession($token);
	        $response->setDataSets(array());
        } catch(\Exception $e){
			$response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
		}
	}

    /***********************************************************************
     * Gets user external id
     ***********************************************************************/
    protected function getUserExternalId($resourceId){
        if(empty($resourceId)){
            return $this->resourceService->getIdentityUserData();
        }
        return $this->entityManager->find(Utilities::ENTITY_DIR.'Resource', $resourceId)->getExternalId();
    }

	public function changeIdentityPassword(Request\DataSet $request, Response $response){
		$row = $request->getDataSet()->getRows();
		$param = $row[0];
		$token = $param['TOKEN'][0];
		$resourceId = empty($param['RESOURCE_ID']) ? null : $param['RESOURCE_ID'];
		try {
            $data = array(
                "requestType" => "Row",
                "row" => array(
                    "PASSWORD" => $param['PASSWORD'],
                    "ID" => $this->getUserExternalId($resourceId)
                )
            );

            if(empty($data['row']['ID'])){
                $response->addMessage(new Message(Messages::ERR_NOT_PROFILE_IDENTITY));
            }

            $user = $this->initService->changeIdentityPassword($data, $token);
            $response->addDataSet(new DataSet('identityPassword', $user));
		} catch(\Exception $e) {
			$response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
			throw $e;
		}
	}

	/**
	 * Creates or saves an user in Identity
	 * @param Request\DataSet $request
	 * @param Response $response
	 * @throws \Exception
     */
	public function saveIdentityUser(Request\DataSet $request, Response $response){
		$row = $request->getDataSet()->getRows();
        $row = $row[0];
		if($row['ID'] != null){
			$external = $this->resourceService->getExternalUserId($row['ID']);
			$row['ID'] = $external[0]['EXTERNAL_ID'];
		} else {
			$row['ID'] = '';
		}
		$password = Utilities::md5CodeIfNecessary($row['PASSWORD']);
		$data = array(
			"requestType" => "DataSet",
			"dataset" => array(array(
				"NAME"      			=> $row['NAME'],
				"FIRSTNAME" 			=> $row['FIRSTNAME'],
				"LASTNAME"  			=> $row['LASTNAME'],
				"EMAIL"     			=> $row['EMAIL'],
				"PASSWORD"  			=> $password,
                "STATUS"                => $row['STATUS'],
                "ID"        			=> $row['ID'],
				"ORGANIZATION_ID" => $this->environment->getCurrentOrganizationId()
			))
		);
		$token = $row['TOKEN'][0];
		try {
            $userInfo = $this->initService->saveUserIdentity($token, $data, $row['EMAIL']);
            if(is_null($userInfo)){
                $response->addMessage(new Response\Message(Messages::ERR_RESOURCE_NOT_IN_IDENTITY, Response\Message::TYPE_ERROR));
                throw new \Exception;
            }
            $response->addDataSet(new DataSet('user', $userInfo));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
			throw $e;
		}
	}

	public function changeIdentityUser(Request\DataSet $request, Response $response){
		$row = $request->getDataSet()->getRows();
		$id = $this->resourceService->getCurrentUserExternalId();
        $row = $row[0];
		$data = array(
			"requestType" => "DataSet",
			"dataset" => array(array(
				"NAME"      => $row['NAME'],
				"FIRSTNAME" => $row['FIRSTNAME'],
				"LASTNAME"  => $row['LASTNAME'],
				"PASSWORD"  => Utilities::md5CodeIfNecessary($row['PASSWORD']),
				"EMAIL"     => $row['EMAIL'],
				"STATUS"     => $row['STATUS'],
				"ID"        => $id[0]['EXTERNAL_ID'],
                "ORGANIZATION_ID" => $this->environment->getCurrentOrganizationId()
				))
		);
		$token = $row['TOKEN'][0];
		try {
			$userInfo = $this->initService->restoreSessionAndCreateUser($token, $data);
			$response->addDataSet(new DataSet('user', $userInfo));
		} catch(\Exception $e) {
			$response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
			throw $e;
		}
	}

    public function getDefaultAclRule() {
        $aclRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\ZhuAccessControlElement');
        $projectRules  = $aclRepository->findBy(array('projectId' => $this->projectId, 'isActive' => 'Y'));
        if(empty($projectRules)) {
            return 'A';
        } else {
            return 'I';
        }
    }
}
